#!/usr/bin/lua

local json = require("luci.json")
local io = require("io")
local fs = require("nixio.fs")
local os = require("os")

local sys = require "luci.sys"
local utl = require "luci.util"
local uci = require "luci.model.uci".cursor()

CODE_SUCCESS                  = 0      -- // ɹ
CODE_NXP_SECURITY_HANDINGSHAKE= 96 	-- // NXP豸ʱ豸ѾͨѶϣڽмֲ
CODE_UPGRADE_STARTED          = 97     -- // ʼ
CODE_UPGRADE_SUCCESS          = 98     -- // ɹ
CODE_WAIT_TO_EXECUTE          = 99     -- // Ѿյȴִ
CODE_UNKNOW_ERROR             = 199    -- // δ֪
CODE_WRONG_FORMAT             = 101    -- // ĸʽ
CODE_UNKNOW_DEVICE            = 102    -- // δ֪豸ָ豸
CODE_UNKNOW_ATTRIBUTE         = 103    -- // δֵ֪
CODE_TIMEOUT                  = 104    -- // ʱ
CODE_BUSY                     = 105    -- // 豸æѾдִеͬʱ
CODE_UNKNDOW_CMD              = 106    -- // δ֪
CODE_NOT_SUPPORTED            = 107    -- // ֵ֧Ĳ
CODE_UPGRADE_MD5SUM_FAILED    = 108    -- // ʱMD5SUMУʧ
CODE_UPGRADE_DOWNLOAD_FAILED  = 109    -- // ʱ̼ʧ
CODE_UPGRADE_FAILED           = 110    -- // ʧ
CODE_PASSWORD_ALREADY_EXISTS  = 111    -- // Ѿ
CODE_PASSWORD_FULL            = 112    -- // Ѿ
CODE_PASSWORD_NOT_EXISTS      = 113    -- // Ҫɾ벻ɹ
CODE_MINUS_1                  = -1     -- // δִҲִеɷ˸

g_device_code	= ""
_gateway_device_id = nil
--: util function-------------------------------------------------------------------

function my_print(s)
	os.execute('logger -s \'' .. os.time() .. ' [INFO] BleCfg::' .. s .. '\'')
end

function uptime() 
	local file=io.open("/proc/uptime", "r")
	if (file == nil) then
		return "0"
	end
	local ut = file:read("*n")
	-- my_print(ut)
	file:close()
	return math.floor(ut)
end
function get_version() 
	local file=io.open("/etc/dusun_build", "r")
	if (file == nil) then
		return "unknown"
	end
	local ver = file:read("*l")
	file:close()
	-- my_print(ver)
	ver = string.gsub(ver, "%s+", "")
	ver = string.gsub(ver, "BUILD_VERSION=", "")
	ver = string.gsub(ver, "_%w+", "")
	return string.sub(ver, 2)
end
function get_model()
	local model=fs.readfile("/tmp/sysinfo/model")
	return  string.gsub(model, "%s+", "")
end
function get_factory()
	return "dusun"
end

function gwmac() 
	local mac=fs.readfile("/sys/class/net/eth0/address")
	return  string.gsub(mac, "%s+", "")
end

function get_devcode() 
	if (g_device_code ~= "") then
		return g_device_code
	end
	local file=io.open("/etc/dusun/device_code")
	if (file ~= nil) then
		g_device_code=fs.readfile("/etc/dusun/device_code")
		file:close()
	end
	return g_device_code
end

function set_devcode(deviceCode) 
	g_device_code = deviceCode
	fs.writefile("/etc/dusun/device_code", deviceCode)
end

function time() 
	return os.time()	
end

function get_gateway_device_id()
	-- cat /proc/sys/kernel/random/uuid
	-- return luci.sys.exec('cat /proc/sys/kernel/random/uuid')
	my_print('gwmac is ' .. gwmac())
	if (_gateway_device_id == nil) then
		local t = io.popen('echo DusunGateway_' .. gwmac() .. ' | md5sum | xargs | cut -d " " -f 1')
		_gateway_device_id = 'ds' .. string.gsub(t:read('*all'), "%s+", "")
		t:close()
	end
	return _gateway_device_id
end
function get_gateway_model() 
	-- return luci.sys.exec('cat /tmp/sysinfo/model')
	-- return get_model()
	return 'hub_ds'
end
function get_gateway_version()
	-- return luci.sys.exec('cat /etc/dusun_build  | head -n1 | xargs | cut -d = -f 2')
	return get_version()
end
function get_gateway_battery() 
	return 100
end
function get_gateway_power_type()
	-- 0 battery, 1 outlet power
	return 1
end
function get_gateway_arm_mode() 
	-- 0 disarm, 1 arm home , 2 arm away
	-- return luci.sys.exec('[ -e /etc/config/dusun/chenbei/bufang ] && cat /etc/config/dusun/chebei/bufang || echo 0')
	local t = io.popen('[ -e /etc/config/dusun/chenbei/bufang ] && cat /etc/config/dusun/chenbei/bufang || echo 0')
	local x = t:read('*all')
	t:close()
	my_print('x is ' .. x)
	return tonumber(string.sub(x, 1, 1))
end
function set_gateway_arm_mode(armMode) 
	-- return luci.sys.call('echo ' .. armMode .. ' > /etc/config/dusun/chenbei/bufang')
	local t = io.popen('echo ' .. armMode .. ' > /etc/config/dusun/chenbei/bufang')
	t:close()
	return 0
end
function get_gateway_sn() 
	-- mac without ':'
	-- return luci.sys.exec('cat /sys/class/net/eth0/address | sed "s/://g"')
	return gwmac()
end
function get_gateway_network_type() 
	-- 4G, wifi, eth
	-- return luci.sys.exec('[ -e /usr/bin/network_type.sh ] && /usr/bin/network_type.sh || echo eth')
	-- return 'eth'
	return string.gsub(luci.util.exec('[ -e /tmp/._network_type ] && cat /tmp/._network_type || echo eth'), "%s+", "")
end
function get_gateway_network_strength() 
	-- a value, 
	-- return luci.sys.exec('[ -e /usr/bin/network_strength.sh ] && /usr/bin/network_strngth.sh || echo 100')
	-- return 100
	return string.gsub(luci.util.exec('[ -e /tmp/._network_strength ] && cat /tmp/._network_strength || echo 100'), "%s+", "")
end
function get_gateway_network_id() 
	-- wifi -> ssid, 4g isp
	-- return luci.sys.exec('[ -e /usr/bin/network_id.sh ] && /usr/bin/network_id.sh || echo 0')
	-- return 'eth'
	return string.gsub(luci.util.exec('[ -e /tmp/._network_id ] && cat /tmp/._network_id || echo eth'), "%s+", "");
end

function get_gateway_cid() 
	-- TODO:
	return 'ds' .. gwmac()
end

--[[
{model = 'Gateway',			devstr = 'Gateway',			sleep = 0,	 fnl = {
	{ fn = 'RemoteSh',					tp = 's'}, 
}},
--]]
dev_model_maps = {
	{model = 'Z3FireSensor',devstr = 'smoke_sensor_ds',			sleep = 0,	 fnl = {}},
	{model = 'Leak',				devstr = 'water_sensor_ds',			sleep = 0,	 fnl = {}},
	{model = 'Dowi',				devstr = 'door_sensor_ds',			sleep = 0,	 fnl = {}},
	{model = 'MoSe 02',			devstr = 'motion_sensor_ds',			sleep = 0,	 fnl = {}},
	{model = 'YRD216 PBDB', devstr = 'lock_actor_ds',			sleep = 0,	 fnl = {}},

	{model = 'IndEnv',			devstr = 'temp_humi_sensor_ds',	sleep = 0,	 fnl = {}},
}

function devstr_search_by_modelstr(modelstr) 
	if (modelstr == nil) then
		return 'UnknownDevice',0
	end
	

	-- my_print('num : ' .. #dev_model_maps)
	
	for i=1,#dev_model_maps do
		local x = dev_model_maps[i]
		local model=x['model']
		-- my_print('model is :' .. model .. ', modelstr is :' .. modelstr)
		if ( model == modelstr ) then
			return x.devstr, x.sleep
		end
	end

	return modelstr,0
end

function devfnl_search_by_modelstr(modelstr) 
	if (modelstr == nil) then
		return {}
	end
	
	-- my_print('num : ' .. #dev_model_maps)
	for i=1,#dev_model_maps do
		local x = dev_model_maps[i]
		local model=x['model']
		-- my_print('model is :' .. model .. ', modelstr is :' .. modelstr)
		if ( model == modelstr ) then
			return x.fnl
		end
	end

	return {}
end

function backup_cmd(fixedHeader, variableHeader, traceId, requestMethod, requestTime, deviceId, deviceModel, data) 
	my_print('backup cmd..')
	local fdir = '/tmp/chenbei/' .. requestMethod .. '/' .. deviceId
	my_print('fdir is ' .. fdir)
	-- /tmp/chenbei/getDeviceInfo/dsb66d9c8e656edcaa1eb6776a5b923a5a
	-- os.execute('mkdir -p ' .. fdir)
	local xcmd = 'mkdir -p ' .. fdir
	my_print('xcmd is ' .. xcmd)
	local t = io.popen(xcmd)
	t:close()
	local s = {
		fixedHeader			= fixedHeader,
		variableHeader	= variableHeader,
		payload					= {
			context				= {
				traceId					= traceId,
				requestMethod		= requestMethod,
				requestTime			= requestTime,
				deviceId				= deviceId,
				deviceModel			= deviceModel,
			},
			data					= data
		}
	}
	t = io.open(fdir .. '/' .. traceId, 'w')
	t:write(json.encode(s))
	t:close()
end

function fetch_one_cmd(requestMethod, deviceId, subDeviceId)
	my_print('fetch one cmd: ' .. requestMethod .. ',deviceId :' .. deviceId)

	local fdir = '/tmp/chenbei/' .. requestMethod .. '/' .. deviceId .. '/'
	local cmd='ls ' .. fdir  .. ' 2>/dev/null | xargs | cut -d " " -f 1'
	local t = io.popen(cmd)
	local y = t:read('*all')
	y = string.gsub(y, "%s+", "")
	t:close()

	if (string.len(y) > 0) then
		local f = fdir .. y
		local t = io.popen('cat ' .. f .. ' 2>/dev/null')
		local s = t:read('*all')
		t:close()
	
		t = io.popen('rm -rf ' .. f)
		t:close()
		my_print(' s is' .. s)

		local js = json.decode(s)
		if (js == nil or js['fixedHeader'] == nil) then
			my_print('nil fixedHeader')
			return nil, nil, nil
		end

		if (js['variableHeader'] == nil) then
			my_print('nil variableHeader')
			return nil, nil, nil
		end

		local payload = js['payload']
		if (payload == nil) then
			my_print('payload is nil')
			return nil, nil, nil
		end

		if (subDeviceId ~= nil) then
			local data = payload['data']
			if (data == nil) then
				my_print('subDeviceId is nil')
				return nil, nil, nil
			end
			local _subDeviceId = data['subDeviceId']
			if (_subDeviceId ~= subDeviceId) then
				return nil, nil, nil
			end
		end

		return js['fixedHeader'], js['variableHeader'], payload['context'], payload['data']
	end

	return nil, nil, nil
end


math.randomseed(os.time())

-- ------------------------------------------------------
function rspcmd(_id, _code) 
	ret =  {
		from		=	"GATEWAY",
		to		=	"CLOUD", 
		mac		=	gwmac(), 
		type		=	"registerResp",
		time		=	time(), 
		deviceCode	=	get_devcode(),
		id		= 	_id,
		data		= 	{
			code		=	_code,
			deviceCode 	= get_devcode(),
			mac		= gwmac()
		}
	}
	return ret
end

function rspnull() 
	return {}
end

function ubus_send(to, msg) 
	--local pkt={PKT = json.encode(msg)}
	local pkt={PKT = msg}
	--my_print('ubus send DS.' .. string.gsub(json.encode(to),'%"', "") .. ' ' .. '\'' .. json.encode(pkt) .. '\'')
	--my_print('ubus send DS.' .. string.gsub(json.encode(to),'%"', "") .. ' ' .. '\'' .. json.encode(pkt) .. '\'')
	--os.execute('ubus send DS.' .. string.gsub(json.encode(to),'%"', "") .. ' \'' .. json.encode(pkt) .. '\'')
	my_print('ubus send DS.' .. to .. ' \'' .. json.encode(pkt) .. '\'')
	--os.execute('ubus send DS.' .. to .. ' \'' .. json.encode(pkt) .. '\'')
	luci.util.exec('ubus send DS.' .. to .. ' \'' .. json.encode(pkt) .. '\'')
end

function gwstatus() 
	ret =  {
		from		=	"GATEWAY",
		to		=	"CLOUD", 
		mac		=	gwmac(), 
		type		=	"reportAttribute",
		time		=	time(), 
		deviceCode	=	get_devcode(),
		data		= 	{
			mac		= gwmac(),
			attribute	= "gateway.status",
			value 		= {
	 			version 	= get_version(),
				model		= get_model(),
				factory		= get_factory(),
				current_time	= time(),
				uptime		= uptime(),
				wireless	= "",
				ethernet_ip	= "",
				uplinkType	= ""
			}
		}
	}
	return ret
end

function gwregister() 
	local devcode = fs.readfile('/proc/sys/kernel/random/uuid')
	devcode = string.gsub(devcode, "%s+", "")
	my_print('devcode is :' .. devcode)
	set_devcode(devcode)
	return {}

	--[[
	return {
		from		=	"GATEWAY",
		to		=	"CLOUD", 
		mac		=	gwmac(), 
		type		=	"registerReq",
		time		=	time(), 
		data		= 	{
			mac		= gwmac(),
			license = get_factory(),
			factory = get_factory(),
		}
	}
	--]]
end

-- cache:
-- --------------------------------------------------------------------------------

-- cache:
-- --------------------------------------------------------------------------------
--[[
__cache = {
	devs = {
		'testdev' = {
			attrs = {
				attr1 = { value = '1', last = 0}
				attr2 = { value = 0,	 last = 1}
			}
		}
	}
	tasks = {
		'testtask' = {
			func		= 'funcname',
			arg			= {},
			start		= start,
			delt		= 5 * 60 * 1000
		}
	}
}
--]]
__cache = {}
function cache_load()
	local __s = fs.readfile("/tmp/._mqtt-gw_cache")
	__cache = json.decode(__s) or {}
end
function cache_save()
	local __s = json.encode(__cache) or '{}'
	fs.writefile("/tmp/._mqtt-gw_cache", __s)
end

-- cache dev
function cache_dev_value_get(dev, attr)
	if (__cache['devs'] == nil) then
		return nil, nil
	end
	if (__cache['devs'][dev] == nil) then
		return nil, nil
	end
	if (__cache['devs'][dev][attr] == nil) then
		return nil, nil
	end
	return __cache['devs'][dev][attr]['value'], __cache['devs'][dev][attr]['last']
end
function cache_dev_value_set(dev, attr, value)
	if (__cache['devs'] == nil) then
		__cache['devs'] = {}
	end
	if (__cache['devs'][dev] == nil) then
		__cache['devs'][dev] = {}
	end
	__cache['devs'][dev][attr] = { value = value, last = time() }
end

-- cache task
function cache_schedule_first_task_delay() 
	local min = nil
	for key, task in pairs(__cache['tasks']) do
		local now = time()
		local diff = task['start'] + task['delt'] - now
		if (min == nil or diff < min) then
			min = diff
		end
	end

	if (min == nil) then
		return nil
	end

	if (min < 0) then
		min = 0
	end

	return min
end
function cache_schedule_add(task,  delt, func, arg) 
	if (__cache['tasks'] == nil) then
		__cache['tasks'] = {}
	end
	__cache['tasks'][task] = {
		func		= func, 
		arg			= arg,
		start		= time(),
		delt		= delt,
		valid		= 1
	}
	local delay = cache_schedule_first_task_delay()
	if (delay == nil) then
		return
	end
	if (delay == 0) then
		delay = 1
	end
	schedule_after(delay)
end
function cache_schedule_del(task)
	if (__cache['tasks'] == nil) then
		return
	end
	__cache['tasks'][task] = nil

	local delay = cache_schedule_first_task_delay()
	if (delay == nil) then
		return
	end
	if (delay == 0) then
		delay = 1
	end
	schedule_after(delay)
end
function cache_schedule_func()
	if (__cache['tasks'] == nil) then
		return {}
	end

	local deltasks = {}
	for key, task in pairs(__cache['tasks']) do
		local now = time()
		local diff = task['start'] + task['delt'] - now
		if (diff < 0) then
			diff = 0
		end

		if (diff == 0) then
			local fname = task['func']
			local f = _G[fname]
			deltasks[key] = '1'
			__cache['tasks'][key]['valid'] = 0
			if (type(fname) == "string" and type(_G) == "table" and type(f) == "function")  then
				f(task['arg'])
			end
		end
	end

	for k,v in pairs(deltasks) do
			if (__cache['tasks'][k]['valid'] == 0) then
				my_print('-- del ' .. k)
				__cache['tasks'][k] = nil
			end
	end

	delay = cache_schedule_first_task_delay()
	if (delay == nil) then
		return {}
	end
	if (delay == 0) then
		delay = 1
	end

	schedule_after(delay)
	return {}
end



--: rpt handler ----------------------------------------------------------------------------
function register_handler(type, from, attr, mac, id, code, value) 
	return {}
end

function cmd_result_handler_add_sub_device(fixedHeader, variableHeader, context, data, type, from, attr, mac, id, code, value) 
	return {}
end
function cmd_result_hanlder(type, from, attr, mac, id, code, value) 
	return {}
end

function report_handler(type, from, attr, mac, id, code, value) 
	my_print('value is ' .. json.encode(value))

	if ( attr == 'gateway.status') then
		return {}
	elseif (attr == 'gateway.state.change') then
		return {}
	elseif (attr == 'gateway.heartbeat') then
		return {}
	elseif (attr == 'gateway.wifi.config.process') then
		local configId	= value['configId']
		local step			= value['step']
		local code			= value['code']
		local msg				= value['msg']
		local debugInfo = value['debugInfo']

		local Protocol			= '01'
		local ServiceType		= '01'
		local SequenceId		= '0001'
		-- bit[7~4]: security, 0000 none, 0001 RSA1024/AES128/CBC-PKCS1Padding
		-- bit[3~0]: Version, 0010
		local Flag					= '12'
		local OpCode				= '8014'
		local StatusCode		= 0
		
		local ret = {
			Protocol		= Protocol,
			ServiceType	= ServiceType,
			SequenceId	= SequenceId,
			Length			= #ret.Payload,
			Flag				= Flag,
			OpCode			= OpCode,
			StatusCode	= 0,
			Payload			= {
				configId	= configId,
				step			= step,
				code			= code,
				msg				= msg,
				debugInfo = debugInfo
			}
		}

		local sret = json.encode(ret)
		my_print(sret)
		tcp_cfg_out('c', sret, #sret)
		return {}
	elseif (attr == 'mod.device_list') then
		return {}	
	elseif (attr == 'mod.new_device_added') then
		return {}
	elseif (attr == 'mod.device_deleted') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		return {}
	elseif (attr == 'device.status') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local battery		= string.format('%d', value['battery'])
		local version		= value['version']
		local online		= string.format('%d', value['online'])
		return {}
	elseif (attr == 'device.zone_status') then
		return {}
	elseif (attr == 'device.temperature') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local unit			= value['unit']
		local valstr		= value['value']

		return {}
	elseif (attr == 'device.humidity') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local unit			= value['unit']
		local valstr		= value['value']
		return {}
	elseif (attr == 'device.meter.energy') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= value['value']
		return {}
	elseif (attr == 'device.meter.power') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= value['value']
		return {}
	elseif (attr == 'device.meter.voltage') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= value['value']
		return {}
	elseif (attr == 'device.meter.current') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= value['value']
		return {}
	elseif (attr == 'device.switch.onoff') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= string.format('%d', value['value'])
		local ep				= value['ep'] or 1
		local fnstr			= string.format("Onoff%d", ep)
		fnstr			= 'Onoff' .. ep
		return {}
	elseif (attr == 'device.meter.onoff') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= string.format('%d', value['value'])
		local ep				= value['ep'] or 1
		local fnstr			= string.format("Onoff%d", ep)
		return {}
	elseif (attr == 'device.light.onoff') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= string.format('%d', value['value'])
		local ep				= value['ep'] or 1
		local fnstr			= string.format("Onoff%d", ep)
		fnstr			= 'Onoff' .. ep
		return {}
	elseif (attr == 'device.onoff') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local valstr		= string.format('%d', value['value'])
		local ep				= value['ep'] or 1
		local fnstr			= string.format("Onoff%d", ep)
		fnstr			= 'Onoff' .. ep
		return {}
	elseif (attr == 'device.lock.check_record') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local passId		= string.format('%d', value['passId'])
		local passType	= string.format('%d', value['passType'])
		local passVal1	= string.format('%d', value['passVal1'])
		local timeval		= string.format('%d', value['time'])
		local pass			= string.format('%d', value['pass'])
		return {}
	elseif (attr == 'device.lock.password_adddel_result') then
		local modelstr	= value['ModelStr']
		modelstr	= devstr_search_by_modelstr(modelstr)
		local operation	= value['operation']
		local passId		= string.format('%d', value['passId'])
		local code			= string.format('%d', value['code'])

		if (passId == '65488') then
			if (code == '0') then
				return {}
			else
				my_print('ignore the 65488 one key open')
				return {}
			end
		else
			return {}
		end
	else 
		my_print('not support attribute :' .. attr)
		return {}	
	end
end

local rpt_handlers = {
	reportAttribute		=  report_handler,
	registerReq				=  register_handler,
	cmdResult					=  cmd_result_hanlder,
}

function rpt_handler_search(type, from, attr, mac, id, value) 
	local rpt_handler = rpt_handlers[type]
	if (rpt_handler == nil) then
		my_print('not support type: ' .. type .. ' message')
		return nil	
	end
	return rpt_handler
end

--: cmd handler implement-------------------------------------------------------------------
--: base64  ----------------------------------------------------------------------------
local b='ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'
function enc(data)
    return ((data:gsub('.', function(x) 
        local r,b='',x:byte()
        for i=8,1,-1 do r=r..(b%2^i-b%2^(i-1)>0 and '1' or '0') end
        return r;
    end)..'0000'):gsub('%d%d%d?%d?%d?%d?', function(x)
        if (#x < 6) then return '' end
        local c=0
        for i=1,6 do c=c+(x:sub(i,i)=='1' and 2^(6-i) or 0) end
        return b:sub(c+1,c+1)
    end)..({ '', '==', '=' })[#data%3+1])
end

function dec(data)
    data = string.gsub(data, '[^'..b..'=]', '')
    return (data:gsub('.', function(x)
        if (x == '=') then return '' end
        local r,f='',(b:find(x)-1)
        for i=6,1,-1 do r=r..(f%2^i-f%2^(i-1)>0 and '1' or '0') end
        return r;
    end):gsub('%d%d%d?%d?%d?%d?%d?%d?', function(x)
        if (#x ~= 8) then return '' end
        local c=0
        for i=1,8 do c=c+(x:sub(i,i)=='1' and 2^(8-i) or 0) end
        return string.char(c)
    end))
end

function random_key_generate(length)
	-- TODO: 
	chs = {
		'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', '0', '1',
	}
	
	local aesKey = ''
	for i=1,16,1 do
		aesKey = aesKey .. chs[math.random(1,16)]
	end

	local aesIv = ''
	for i=1,16,1 do
		aesIv = aesIv .. chs[math.random(1,16)]
	end
	
	--[[
	local aesKey = '1111111111111111'
	local aesIv	 = '2222222222222222'
	--]]

	local aesKey_b64 = enc(aesKey)
	local aesIv_b64  = enc(aesIv)


	return aesKey_b64, aesIv_b64
end

function password_mask(psw) 
	local iS     = string.byte(" ", 1)
	local iA     = string.byte("A", 1)
	local iZ     = string.byte("Z", 1)
	local pswMask = ''

	for i=1,string.len(psw) do
		local ix     = string.byte(psw, i)
		if (i == 1) then
			if (ix >= iA and ix <= iZ) then
				pswMask = pswMask .. '_'
			elseif (ix == iS) then
				pswMask = pswMask .. ' '
			else
				pswMask = pswMask .. '*'
			end
		else
			if (ix == iS) then
				pswMask = pswMask .. ' '
			else
				pswMask = pswMask .. '*'
			end
		end
	end
	return pswMask
end


function set_wifi_mtksdk(data, traceId)
	--{"configId": "1ce82026-b1ac-4388-8126-d2313a25bcee", "ssid": "AAAAAA", "pwd": "dl123456", "chan": 6}
	local ssid			= data['ssid'] or '______'
	local pwd			= data['pwd'] or '______'
	local chan			= data['chan']
	local configId		= data['configId'] or "noconfigId"
	local sdn           = data['serverDN']

	if (ssid == '') then
		ssid = '______'
	end
	if (pwd == '') then
		pwd = '______'
	end
	

	local flag = 0
	local i = 0


	while i < 5 do
		my_print('--try ' .. i)

		local fret = iwpriv(ssid)
		if (fret == 0) then
			iwsave(ssid)
			flag = 1
			break;
		end
	
		i = i + 1
	end

	--luci.util.exec('ifconfig ra0 down')
	local OpCode			= '8014'
	local SequenceId		= '0001'
	local code = 0
	local msg = 'OK'
	local debugInfo = {}
	ret = {
		SequenceId		= SequenceId,
		OpCode			= OpCode,
		Payload			= {
			configId		= configId,
			step			= 'DEV_RECEIVE_APP_DATA',
			code			= code,
			msg				= msg,
			debugInfo = {
			}
		}
	}
	
	ubus_send('BLE.NETCFG.RSP',ret)  -- Ϣble
	
	if (flag == 0) then
		local pswMd5	= luci.util.exec('echo ' .. pwd .. ' | md5sum | awk \'{printf "%s",$1}\'')
		local pswMask = password_mask(pwd)

		local SequenceId		= '0001'

		local OpCode			= '8014'

		local code = 52   -- ERR_CONFIG_NO_AP_FOUND	ʱ豸ҲWiFiȵ
		local msg = 'ERR_CONFIG_NO_AP_FOUND'
		local debugInfo = {	
			ssid = ssid
		}
		ret = {
			SequenceId	= SequenceId,
			OpCode			= OpCode,
			Payload			= {
				configId	= configId,
				step		= 'DEV_CONNECTED_WIFI',
				code		= code,
				msg			= msg,
				debugInfo = {
					ssid	= ssid,
					pswMd5	= pswMd5,
					pswMask	= pswMask
				}
			}
		}
		
		ubus_send('BLE.NETCFG.RSP',ret)  -- Ϣ ble
		return 0
	end
		
	my_print('-- set wifi mtksdk ')
	-- do set wifi,  -- λָ
	-- my_print('setting ssid:"' .. ssid .. '",pwd:"' .. pwd .. '",chan:' .. chan)
	luci.util.exec('uci set /etc/config/apcli.sta.ssid="' .. ssid .. '";uci set /etc/config/apcli.sta.password="' .. pwd .. '";uci commit /etc/config/apcli;')

	-- luci.util.exec('echo ' .. configId .. ' > /tmp/._set_wifi_ap_configId')
	luci.util.exec('echo ' .. configId .. ' > /tmp/._set_wifi_ble_configId')
	luci.util.exec('echo ' .. configId .. ' > /tmp/._set_wifi_configId')
	luci.util.exec('echo ' .. sdn .. ' > /tmp/._cloud_sdn')
	
	-- ״̬ļ
	luci.util.exec('echo ' .. sdn .. ' > /tmp/._ble_netcfg')

	luci.util.exec('/etc/init.d/ap_clientd stop;/etc/init.d/ap_clientd start;')
	-- cache_schedule_add('setWifiApExpiredTask', 10, 'set_wifi_ap_expired_func', 10)
	cache_schedule_add('setWifiBleExpiredTask', 10, 'set_wifi_ble_expired_func', 10)  -- 
	cache_schedule_add('setWifiExpiredTask', 10, 'set_wifi_expired_func', 10)

	return 0
end


function set_wifi(data, traceId)
	-- v0.0.2
	-- [[
	wifiSSid	= data['wifiSsid']
	wifiPwd		=	data['wifiPwd']	
	--]]
	
	-- v0.0.4
	local ssid			= data['ssid']
	local pwd				= data['pwd']
	local chan			= data['chan']

	-- do set wifi
	print('setting ssid:"' .. ssid .. '",pwd:"' .. pwd .. '",chan:' .. chan)

	local n = 0                                               
        local wn=""                                               
        uci:foreach("wireless", "wifi-iface",               
                function (s)                                                       
                        if n == 0 then                                             
                                wn = s['.name']                                    
                        end                                                        
                        n = n + 1                                                  
		end
	)
	uci:set('wireless', wn, 'ssid', ssid)
	uci:set('wireless', wn, 'encryption', 'psk2+ccmp')
	uci:set('wireless', wn, 'key',	      pwd)
	uci:set('wireless', wn, 'network', 'wan1')
	uci:set('wireless', wn, 'mode', 'sta')
	uci:set('wireless', wn, 'disabled', '0')
  uci:commit("wireless")

  uci:set("network", "wan1", "interface")
  uci:set("network", "wan1", "proto", "dhcp")
  uci:commit("network")

  n=0
  local fn=""
  uci:foreach("firewall", "zone",
		function (s)
		 if n == 1 then
		  fn = s['.name']
     end
     n = n + 1
    end
  )
	local nks=uci:get_list("firewall", fn, "network")
  for k, v in pairs(nks) do
		if v == "wan1" then
		 	table.remove(nks, k)
    end
  end
  table.insert(nks, "wan1")
  -- luci.util.dumptable(nks)
  uci:set_list("firewall", fn, "network", nks)
  uci:commit("firewall")

  os.execute("/etc/init.d/firewall reload 2>1 1>/dev/null")
  --os.execute("wifi reload wlan0")
  os.execute("wifi&")

	local ubus_msg = {
			to		= 'GATEWAY',
			from	= 'NXP',
			type	= 'reportAttribute',
			data	= {
				mac	= gwmac(),
				attribute		= 'gateway.wifi.config.process',
				value				= {
					configId	= traceId,
					step			= 'RECEIVE_CONFIG',
					code			= 0,
					msg				= 'OK',
					debugInfo = {
						yum = 'yum'
					}
				}
			},
			time				= time(),
			deviceCode	= get_devcode(),
			mac					= gwmac()
	}
	ubus_send('GATEWAY', ubus_msg)
		
	ubus_msg['data']['step'] = 'CONNECT_WIFI'
	ubus_send('GATEWAY', ubus_msg)
	
	luci.util.exec('echo ' .. traceId .. ' > /tmp/._set_wifi_ap_configId')

	return 0
end
function quanity2rssi(quanity) 
	if (quanity <= 0) then
		return -100
	elseif (quanity >= 100) then
		return -50
	else
		return math.floor((quanity/2) - 100)
	end
end

function get_wifi_list_mtksdk() 
	local __s = fs.readfile("/tmp/._wifilist")
	local __s = iwscan('')
	local wflist = json.decode(__s) or {}

	local wl = {}
	for i=1,#wflist do
		if (wflist[i]['ssid'] ~= ' ' and wflist[i]['ssid'] ~= '') then
			wl[#wl+1] = {}

			wl[#wl]['SSID'] = wflist[i]['ssid']
			--wl[#wl]['rssi'] = 0 - tonumber(wflist[i]['Signal'])
			--wl[#wl]['rssi'] = 0 - tonumber(wflist[i]['rssi'])
			wl[#wl]['RSSI'] = quanity2rssi(tonumber(wflist[i]['rssi']))
			wl[#wl]['chan'] = tonumber(wflist[i]['Channel'])

			local auth_int = 0
			-- WPAPSK WPA2PSK		WPAPSKWPA2PSK
			local security = wflist[i]['security'] 
			-- AES TKIP TKIPAES
			local crypto	 = wflist['crypto']
			
			if (security == 'NONE') then
				auth_int = 0
			elseif (security == 'WPAPSK') then
				auth_int = 2
			elseif (security == 'WPA2PSK') then
				auth_int = 3
			elseif (security == 'WPA1PSKWPA2PSK') then
				auth_int = 4
			else
				auth_int = 5
			end

			wl[#wl]['AUTH'] = auth_int
		end
	end

	return 0, wl
end
function get_wifi_list()
	--[[
	s = luci.sys.exec('iwpriv ra0 set SiteSurvey=0; iwpriv ra0 get_site_survey | awk \'{ if (NR > 2 && NF == 9) print "{ wifiSsid = "$2", wifiRssi = "$5", wifiAuth = "$4" }" }\'')
	my_print(s)
	return 0, json.encode(s)
	-- iwlist  wlan1 scanning | grep "Channel:\|ESSID:\|Signal\|Group Chpher\|Pairwise Ciphers\|Authentication Suites\|WPA" | sed "s/:/ /g" | sed "s/level=//g"| awk '{if (NR > 1 && $1 == "Channel") print ","; if($1==
"Channel") print "{chan = "$2","; else if ($2 == "Signal") print "rssi = "$3","; else if($1 == "ESSID") print "ssid = "$2",auth=AUTH_"; else if (match($2,"WPA")) print "WPA"; else if(match($3,"WPA2")) print "WPA2"; else if(
$1 == "Authentication") print $4"}" }'
	--]]
	
	-- v0.0.2
	--[[
	return 0, {{
			wifiSsid	= 'network1',
			wifiRssi	= -5,	
			wifiAuth	= 3
	},{
			wifiSsid	= 'network2',
			wifiRssi	= -10,	
			wifiAuth	= 3
	}}
	--]]

	local wfenable = uci:get('wireless', 'radio0', 'disabled')
	if (wfenable == '1') then
		uci:set('wireless', 'radio0', 'disabled', '0')
		uci:commit('wireless')
		os.execute('wifi&;sleep 8')
	end

	-- ps=luci.util.execi('iw dev wlan1 scan')
	local ps=luci.util.execi('iwlist wlan1 scanning')
	if not ps then
        return 0, {}
	else
        ps()
	end

	local wflist = {}
	for i in ps do
		local s,e = string.find(i, 'Cell.+Address:+')
    if (s ~= nil) then
			wflist[#wflist+1] = {
				Channel = 0,
				SSID    = '',
				Signal  = 0,
				WPA     = 0,
				WPA2    = 0,
				PSK     = 0,
				PSK2    = 0
			}
    end
		s,e = string.find(i, 'Channel:[%d]+')
		if (s ~= nil) then
		   -- print(string.sub(i, s + 8, e))
       wflist[#wflist]['Channel'] = string.sub(i, s+8, e)
    end

    s,e = string.find(i, 'ESSID:"[%S]+"')
    if (s ~= nil) then
       -- print(string.sub(i, s + 7, e-1))
       wflist[#wflist]['SSID'] = string.sub(i, s+7, e-1)
    end

    s,e = string.find(i, 'Signal level=[-%d]+')
    if (s ~= nil) then
       -- print(string.sub(i, s + 14, e) .. ' dBm')
       wflist[#wflist]['Signal'] = string.sub(i, s+14, e)
    end

    s,e = string.find(i, 'IE:.+WPA2 Version')
    if (s ~= nil) then
       wflist[#wflist]['WPA2'] = 1
    end

    s,e = string.find(i, 'IE:.+WPA Version')
    if (s ~= nil) then
       wflist[#wflist]['WPA'] = 1
    end

    s,e = string.find(i, 'Authentication Suites.+PSK2')
    if (s ~= nil) then
       wflist[#wflist]['PSK2'] = 1
    end

    s,e = string.find(i, 'Authentication Suites.+PSK')
    if (s ~= nil) then
       wflist[#wflist]['PSK'] = 1
    end
	end


	-- luci.util.dumptable(wflist)
	local wl = {}
	for i=1,#wflist do
		-- print('-------------')
		-- luci.util.dumptable(wflist[i])
		if (wflist[i]['SSID'] ~= '') then
			wl[#wl+1] = {}

			wl[#wl]['ssid'] = wflist[i]['SSID']
			wl[#wl]['rssi'] = 0 - tonumber(wflist[i]['Signal'])
			wl[#wl]['chan'] = tonumber(wflist[i]['Channel'])

			-- WPA WPA2 PSK PSK2
			local auth = 'AUTH'
			if ( wflist[i]['WPA'] == 1 and wflist[i]['WPA2'] == 1 ) then
				auth = auth .. '_WPA_WPA2'
			elseif (wflist[i]['WPA'] == 1) then
				auth = auth .. '_WPA'
			elseif (wflist[i]['WPA2'] == 1) then
				auth = auth .. '_WPA2'
			else
				auth = auth .. '_NONE'
			end

			if (auth ~= 'AUTH_NONE') then
				if (wflist[i]['PSK'] == 1) then
					auth = auth .. '_PSK'
				elseif (wflist[i]['PSK2'] == 1) then
					auth = auth .. '_PSK2'
				end
			end

			local auth_int = 0
			if (auth == 'AUTH_NONE') then
				auth_int = 0
			elseif (auth == 'AUTH_WPA_PSK') then
				auth_int = 2
			elseif (auth == 'AUTH_WPA2_PSK') then
				auth_int = 3
			elseif (auth == 'AUTH_WPA_WPA2_PSK') then
				auth_int = 4
			elseif (auth == 'AUTH_WPA_PSK2') then
				auth_int = 2
			elseif (auth == 'AUTH_WPA2_PSK2') then
				auth_int = 3
			elseif (auth == 'AUTH_WPA_WPA2_PSK2') then
				auth_int = 4
			elseif (auth == 'AUTH_WPA') then
				auth_int = 2
			elseif (auth == 'AUTH_WPA2') then
				auth_int = 3
			elseif (auth == 'AUTH_WPA_WPA2') then
				auth_int = 4
			else 
				auth_int = 5
			end

			wl[#wl]['auth'] = auth_int
		end
	end

	-- luci.util.dumptable(wl)
	
	return 0, wl
	
	-- v0.0.4
	-- auth - 0->auth_open, 1->auth_wep, 2->auth_wap_psk, 3->auth_wpa2_psk, 4->auth_wpa_wpa2_psk, 5->auth_max
	--[[
	return 0, {{
		ssid			= 'network1',
		rssi			= -5,
		auth			= 0,
		chan			= 11
	}, {
		ssid			= 'network1',
		rssi			= -10,
		auth			= 0,
		chan			= 12
	}}
	--]]
end

--: cmd handler ----------------------------------------------------------------------------
function cmd_key_swap(SequenceId,OpCode, Payload)
	local publicRsaKey = Payload['key']

	-- if (publicRsaKey == nil or string.len(publicRsaKey) ~= 128) then
	if (publicRsaKey == nil) then
		my_print('rsa public key error')
		return {}
	end
	
	-- base64 format
	local aesKey,aesIv		= random_key_generate(16)

	local ret = {
		SequenceId	= SequenceId,
		OpCode			= OpCode,
		Payload			= {
			key = aesKey,
			iv	= aesIv
		}
	}
	return ret
end

function remote_config_wifi_func(data) 
	set_wifi(data, 'traceId')
end
function cmd_set_wifi(SequenceId, OpCode,Payload)
	local configId = Payload['configKey']
	ssid		 = Payload['wifiSSID']
	pwd			 = Payload['wifiPassword']
	serverDN     = Payload['serverDN']

	local code = 0
	local ret = {
		SequenceId	= SequenceId,
		OpCode		= OpCode,
		Payload		= {
		}
	}
	
	ubus_send('BLE.NETCFG.RSP',ret)

	local data = { 
		ssid		= ssid,
		pwd			= pwd,
		chan		= chan,
		configId	= configId,
		serverDN	= serverDN
	}
	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	if (mtksdk == '1') then
		set_wifi_mtksdk(data, configId)
	else
		--set_wifi(Payload, configId)
		cache_schedule_add('RemoteConfigWifi', 4, 'remote_config_wifi_func', data)
	end

	return {}
end
function cmd_get_wifi_list(SequenceId, OpCode,Payload)
	local page			= Payload['page']
	local needScan	= Payload['scan']
	my_print('--- 1')

	local mtksdk=luci.util.exec('[ -e /tmp/sysinfo/mtksdk ] && echo -n 1 || echo -n 0')
	local code = 0
	local wifiList = {}
	if (mtksdk == '1') then
		code, wifiList	= get_wifi_list_mtksdk()
	else
		code, wifiList	= get_wifi_list()
	end


	my_print('--- 2')
	my_print('code is ' .. code .. ',wifilist is ' .. json.encode(wifiList))

	local ret = {
		SequenceId	= SequenceId,
		OpCode			= OpCode,
		Payload			= {
			result		= code,
			totalPage	= 1,
			currentPage = 1,
			wifiList	= wifiList
		}
	}
	
	--ubus_send('BLE.NETCFG.RSP',ret)
	
	return ret
end
function cmd_get_device_info(SequenceId, OpCode, Payload)

	--local dgstr  = dginfo('a')
	--local dgjson = json.decode(dgstr)
	
	local status = 0
	if(get_gateway_network_type() == 'wifi') then
		status = 1
	end
	local deviceType   = 'hub'
	local deviceRegion =  'US'
	local ret = {
		SequenceId	= SequenceId,
		OpCode			= OpCode,
		Payload			= {
			firmVersion		= get_gateway_version(),
			cid				= get_gateway_device_id(),
			model			= get_model(),
			networkStatus   = status,
			deviceMac		= gwmac(),
			deviceType      = deviceType,
			deviceRegion    = deviceRegion
			--rfstatus		= dgjson
		}
	}

	return ret
end
function cmd_cancle_config_wifi(SequenceId, OpCode, Payload)
	-- 0 ok, 1 except
	local code = 0
	local ret = {

		SequenceId	= SequenceId,
		OpCode		= OpCode,
		Payload		= {
			result		= code
		}
	}

	return ret
end

function cmd_query_log(SequenceId, OpCode, Payload)
	-- 0 ok, 1 except
	slogList = {}
	--my_print('slog is ' .. slogList)
	logList = json.decode(slogList)
	local ret = {

		SequenceId	= SequenceId,
		OpCode		= OpCode,
		Payload		= {
			logList = logList
		}
	}

	return ret

end


--[[
function cmd_query_debug(Protocol, ServiceType, SequenceId, Length, Flag, OpCode, StatusCode, Payload)
	-- 0 ok, 1 except
	local dgstr = dginfo('a')
	my_print('dbginfo:' .. dgstr)

	local dgjson = json.decode(dgstr)
	local appver = luci.util.exec('cat /etc/config/dusun/mqtt-gw/version 2>/dev/null | xargs | cut -d "," -f 1 | cut -d ":" -f 2')

	local ret = {
		Protocol		= Protocol,
		ServiceType	= ServiceType,
		SequenceId	= SequenceId,
		Length			= 0,
		Flag				= Flag,
		OpCode			= OpCode,
		StatusCode	= 0,
		Payload			= {
			firmVersion		= appver,
			cid						= get_gateway_device_id(),
			model					= get_gateway_model(),
			networkStatus	= 0,
			deviceMac			= gwmac(),
			rfstatus			= dgjson
		}
	}
	ret.Length			= #json.encode(ret.Payload)

	local sret = json.encode(ret)
	my_print(sret)
	tcp_cfg_out('c', sret, #sret)
	return {}
end
--]]


--  0x8014			= rpt_wifi_connect_status,
local cmd_handlers = {
	hub_ds					= { 
		h8000			= cmd_key_swap,
		h8012			= cmd_set_wifi,
		h8011			= cmd_get_wifi_list,
		h8010			= cmd_get_device_info,
		h8016			= cmd_cancle_config_wifi,
		h8017			= cmd_query_log
	}
}
function cmd_handler_search(deviceModel, requestMethod) 
	local cmd_handler_map = cmd_handlers[deviceModel]
	if (cmd_handler_map == nil) then
		my_print('not support device model : ' .. deviceModel)
		return nil
	end

	_requestMethod = 'h' .. requestMethod
	local command_handler = cmd_handler_map[_requestMethod]
	if (command_handler == nil) then
		my_print('not support method:' .. requestMethod)
		return nil
	end

	return command_handler
end

--: Hanlder For Msg And Cmd------------------------------------------------------------------
function handler_msg(jmsg) 
	local type = jmsg['type']
	if (type == nil) then
		my_print('no type')
		return rspnull()
	end

	local from	= jmsg['from']
	if (from == nil) then
		my_print('no from')
		return rspnull()
	end

	local data = jmsg['data']
	if (data == nil) then
		my_print('no type')
		return rspnull()
	end
	
	local attr	= data['attribute'] or ''
	local mac		= data['mac'] or ''
	local id		= data['id'] or ''
	local code	= data['code'] or ''
	local value = data['value'] or ''

	my_print('type:[' .. type .. '],from:[' .. from .. '],attr:[' .. attr .. '],mac:[' .. mac .. ']\n')
	local rpt_handler = rpt_handler_search(type, from, attr, mac)
	if (rpt_handler == nil) then
		my_print('not support msg!')
		return rspnull()
	end

	my_print('execute rpt ' .. type .. ' handler')
	return rpt_handler(type, from, attr, mac, id, code, value)
end

function handler_cmd(jcmd) 

	local SequenceId	= jcmd['SequenceId']
	if (SequenceId == nil) then
		my_print('error SequenceId')
		return rspnull()
	end

	
	local OpCode			= jcmd['OpCode']
	if (OpCode == nil) then
		my_print('error OpCode')
		return rspnull()
	end

	
	local Payload			= jcmd['Payload']
	if (Payload == nil ) then
		my_print('error Payload')
		return rspnull()
	end

	payload = Payload

	my_print('OpCode:[' .. OpCode  .. ']')

	
	local cmd_handler=cmd_handler_search('hub_ds', OpCode)
	if (cmd_handler == nil) then
		my_print('not support cmd:' .. OpCode)
		local ret = {
			err             = 'unknowed opcode',
			SequenceId		= SequenceId,
			OpCode			= OpCode,
			Payload			= {
			}
		}

		return ret
	end

	my_print('cmd handler for ' .. OpCode)
	return cmd_handler(SequenceId, OpCode, payload)
end

--: Call From Exe--------------------------------------------------------------------------------
function msgin(msg) 
	local jmsg = json.decode(msg)
	
	my_print('jmsg is' .. msg)
	
	local jret, topic = handler_msg(jmsg)
	
	local sret = json.encode(jret)

	if ( #sret > 6 ) then
		if (topic == nil) then
			topic = 'report'
		end

		my_print('publish to ' .. topic .. ': ' .. sret)

		mqtt_publish(topic, sret, #sret)
	end

	return '{}'
end

function cmdin(cmd) 
	cache_load()

	local jcmd = json.decode(cmd)
	
	my_print('jcmd is' .. cmd)
	
	local jret = handler_cmd(jcmd)

	local sret = json.encode(jret)
	my_print('sret is :' .. sret)

	--topic = 'report'
	--mqtt_publish(topic, sret, #sret)
		
	cache_save()
	return sret
end

function callf(fname) 
	if (type(fname) ~= "string")  then
		return json.encode(rspnull())
	end
	if (type(_G) ~= "table") then
		return json.encode(rspnull())
	end
	local f = _G[fname]

	--[[
	for v in fname:gmatch("[^%.]+") do
		if (type(f) ~= "table") then
			return json.encode(rspnull())
		end
		f=f[v]
	end
	--]]

	if (type(f) ~= "function") then
		return json.encode(rspnull())
	end
	
	s=json.encode(f())
	-- my_print(s)
	return s
end



--TODO:
-- 1. fixed the TODO in this file
--[[
AES IV: 7E668ED367E52FF196CA0A820A0EB96F
AES Key: 74C470FFBAE6E5D2E7F352039E4D70FB
Algorithm: AES/CBC/PKCS7Padding
JSON: {"key":"-----BEGIN PUBLIC KEY-----\nMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQCKoDeCtt0KFTCHBN\/wN8I3qJjK\n3MLmqbuY0R51FunGYSDJruYVbRy\/0HJ9QEXcpQ6HsOAKLW+xoJasNGkAxA2QCLQB\nu+hETVNMkyY4BIIxOGe\/0qdfPp8cxhxg1XBVPL84RO\/hPukdm6zZRbh4JE+ldbhT\nNQ1pJ\/UO3HphfPv84wIDAQAB\n-----END PUBLIC KEY-----\n"}

Raw Bytes: 0x7B,0x22,0x6B,0x65,0x79,0x22,0x3A,0x22,0x2D,0x2D,0x2D,0x2D,0x2D,0x42,0x45,0x47,0x49,0x4E,0x20,0x50,0x55,0x42,0x4C,0x49,0x43,0x20,0x4B,0x45,0x59,0x2D,0x2D,0x2D,0x2D,0x2D,0x5C,0x6E,0x4D,0x49,0x47,0x66,0x4D,0x41,0x30,0x47,0x43,0x53,0x71,0x47,0x53,0x49,0x62,0x33,0x44,0x51,0x45,0x42,0x41,0x51,0x55,0x41,0x41,0x34,0x47,0x4E,0x41,0x44,0x43,0x42,0x69,0x51,0x4B,0x42,0x67,0x51,0x43,0x4B,0x6F,0x44,0x65,0x43,0x74,0x74,0x30,0x4B,0x46,0x54,0x43,0x48,0x42,0x4E,0x5C,0x2F,0x77,0x4E,0x38,0x49,0x33,0x71,0x4A,0x6A,0x4B,0x5C,0x6E,0x33,0x4D,0x4C,0x6D,0x71,0x62,0x75,0x59,0x30,0x52,0x35,0x31,0x46,0x75,0x6E,0x47,0x59,0x53,0x44,0x4A,0x72,0x75,0x59,0x56,0x62,0x52,0x79,0x5C,0x2F,0x30,0x48,0x4A,0x39,0x51,0x45,0x58,0x63,0x70,0x51,0x36,0x48,0x73,0x4F,0x41,0x4B,0x4C,0x57,0x2B,0x78,0x6F,0x4A,0x61,0x73,0x4E,0x47,0x6B,0x41,0x78,0x41,0x32,0x51,0x43,0x4C,0x51,0x42,0x5C,0x6E,0x75,0x2B,0x68,0x45,0x54,0x56,0x4E,0x4D,0x6B,0x79,0x59,0x34,0x42,0x49,0x49,0x78,0x4F,0x47,0x65,0x5C,0x2F,0x30,0x71,0x64,0x66,0x50,0x70,0x38,0x63,0x78,0x68,0x78,0x67,0x31,0x58,0x42,0x56,0x50,0x4C,0x38,0x34,0x52,0x4F,0x5C,0x2F,0x68,0x50,0x75,0x6B,0x64,0x6D,0x36,0x7A,0x5A,0x52,0x62,0x68,0x34,0x4A,0x45,0x2B,0x6C,0x64,0x62,0x68,0x54,0x5C,0x6E,0x4E,0x51,0x31,0x70,0x4A,0x5C,0x2F,0x55,0x4F,0x33,0x48,0x70,0x68,0x66,0x50,0x76,0x38,0x34,0x77,0x49,0x44,0x41,0x51,0x41,0x42,0x5C,0x6E,0x2D,0x2D,0x2D,0x2D,0x2D,0x45,0x4E,0x44,0x20,0x50,0x55,0x42,0x4C,0x49,0x43,0x20,0x4B,0x45,0x59,0x2D,0x2D,0x2D,0x2D,0x2D,0x5C,0x6E,0x22,0x7D
Secure: 0x92,0x72,0x71,0xF2,0x7B,0x72,0x33,0x5B,0x61,0xA2,0x08,0xB7,0x4D,0xC8,0xB4,0x80,0x43,0xF6,0xAA,0xAC,0xC6,0x96,0xC0,0xA7,0x5E,0xE7,0x58,0x80,0xBD,0xBC,0x3A,0x80,0x47,0x95,0x8E,0xC6,0x20,0x37,0x5C,0x45,0x49,0xDC,0x82,0x68,0xCE,0x14,0xF8,0x1B,0xA5,0xA4,0x78,0x3F,0x1A,0x96,0x8E,0xBA,0xE3,0x41,0x6D,0x45,0xA8,0xE1,0x70,0xD1,0x28,0x70,0x78,0xDB,0x7D,0x0E,0x1E,0x8B,0x59,0x6C,0xF1,0xB8,0xA3,0xE2,0x82,0x59,0xA2,0xAB,0x62,0x99,0xF0,0x7C,0x06,0x4D,0x61,0x14,0x27,0x68,0x49,0x60,0x72,0x1D,0x11,0x16,0xEF,0x2B,0xCB,0xCC,0x98,0x2D,0x3C,0x89,0x47,0xC7,0x41,0xA4,0x65,0xDB,0x30,0x68,0x35,0x04,0xA3,0x61,0xB1,0x8A,0x0B,0x50,0xB9,0xD4,0x20,0xDA,0xB5,0xFA,0xDF,0x42,0x95,0x6F,0xF3,0x19,0xF1,0x02,0xE7,0xEE,0x2D,0xC4,0x87,0x6C,0xBF,0xB8,0x36,0x36,0x08,0x41,0x10,0x63,0xAE,0xCE,0xBA,0x6E,0x6E,0x76,0xCC,0x69,0xC2,0x81,0x5E,0xBC,0x89,0xBC,0x90,0x44,0xF8,0x15,0x2E,0xE7,0xBD,0x93,0x71,0x0F,0x18,0x2E,0x3C,0x44,0x74,0xDA,0x52,0xFB,0x83,0x22,0x3F,0x25,0xAD,0x18,0x01,0x29,0x18,0x92,0x8D,0xB1,0xE3,0xFE,0x35,0x36,0x4F,0xC4,0x6D,0x96,0x93,0x8D,0xB1,0x84,0x3B,0x00,0x22,0xF4,0xB5,0x2B,0x00,0xA1,0x29,0xED,0xFB,0x58,0x01,0xB5,0x8F,0x28,0x3B,0x62,0xD1,0x66,0x0B,0xF2,0xF5,0xB6,0x15,0x13,0xC7,0x3D,0xE6,0x99,0xE8,0xD5,0x3C,0x8B,0x43,0xA7,0xEF,0x23,0x5A,0x94,0x68,0xE9,0x0D,0x03,0x32,0xED,0xAA,0x8A,0xFC,0x65,0xB9,0x8C,0x04,0x46,0x01,0x08,0x99,0x5C,0x56,0xB0,0x88,0x14,0xAE,0x36,0x67,0xFB,0x92,0x19,0x97,0x69,0x3A,0x63,0xE4,0x89,0x2C,0x3A,0x52,0x1E,0x7E,0x32,0xA9,0xC1,0x4A,0xC2,0xFE,0xA8,0x93,0x20,0xEE,0x1A,0x55,0x53,0xB7,0x67,0x40,0x92,0x5D,0x63
Full Request:0x01,0x01,0x00,0x01,0x00,0x00,0x01,0x34,0x12,0x80,0x00,0x00,0x92,0x72,0x71,0xF2,0x7B,0x72,0x33,0x5B,0x61,0xA2,0x08,0xB7,0x4D,0xC8,0xB4,0x80,0x43,0xF6,0xAA,0xAC,0xC6,0x96,0xC0,0xA7,0x5E,0xE7,0x58,0x80,0xBD,0xBC,0x3A,0x80,0x47,0x95,0x8E,0xC6,0x20,0x37,0x5C,0x45,0x49,0xDC,0x82,0x68,0xCE,0x14,0xF8,0x1B,0xA5,0xA4,0x78,0x3F,0x1A,0x96,0x8E,0xBA,0xE3,0x41,0x6D,0x45,0xA8,0xE1,0x70,0xD1,0x28,0x70,0x78,0xDB,0x7D,0x0E,0x1E,0x8B,0x59,0x6C,0xF1,0xB8,0xA3,0xE2,0x82,0x59,0xA2,0xAB,0x62,0x99,0xF0,0x7C,0x06,0x4D,0x61,0x14,0x27,0x68,0x49,0x60,0x72,0x1D,0x11,0x16,0xEF,0x2B,0xCB,0xCC,0x98,0x2D,0x3C,0x89,0x47,0xC7,0x41,0xA4,0x65,0xDB,0x30,0x68,0x35,0x04,0xA3,0x61,0xB1,0x8A,0x0B,0x50,0xB9,0xD4,0x20,0xDA,0xB5,0xFA,0xDF,0x42,0x95,0x6F,0xF3,0x19,0xF1,0x02,0xE7,0xEE,0x2D,0xC4,0x87,0x6C,0xBF,0xB8,0x36,0x36,0x08,0x41,0x10,0x63,0xAE,0xCE,0xBA,0x6E,0x6E,0x76,0xCC,0x69,0xC2,0x81,0x5E,0xBC,0x89,0xBC,0x90,0x44,0xF8,0x15,0x2E,0xE7,0xBD,0x93,0x71,0x0F,0x18,0x2E,0x3C,0x44,0x74,0xDA,0x52,0xFB,0x83,0x22,0x3F,0x25,0xAD,0x18,0x01,0x29,0x18,0x92,0x8D,0xB1,0xE3,0xFE,0x35,0x36,0x4F,0xC4,0x6D,0x96,0x93,0x8D,0xB1,0x84,0x3B,0x00,0x22,0xF4,0xB5,0x2B,0x00,0xA1,0x29,0xED,0xFB,0x58,0x01,0xB5,0x8F,0x28,0x3B,0x62,0xD1,0x66,0x0B,0xF2,0xF5,0xB6,0x15,0x13,0xC7,0x3D,0xE6,0x99,0xE8,0xD5,0x3C,0x8B,0x43,0xA7,0xEF,0x23,0x5A,0x94,0x68,0xE9,0x0D,0x03,0x32,0xED,0xAA,0x8A,0xFC,0x65,0xB9,0x8C,0x04,0x46,0x01,0x08,0x99,0x5C,0x56,0xB0,0x88,0x14,0xAE,0x36,0x67,0xFB,0x92,0x19,0x97,0x69,0x3A,0x63,0xE4,0x89,0x2C,0x3A,0x52,0x1E,0x7E,0x32,0xA9,0xC1,0x4A,0xC2,0xFE,0xA8,0x93,0x20,0xEE,0x1A,0x55,0x53,0xB7,0x67,0x40,0x92,0x5D,0x63
--]]
